'' =================================================================================================
''
''   File....... jm_jack-o-lantern.spin
''   Purpose.... 
''   Author..... Jon "JonnyMac" McPhalen
''               Copyright (c) 2010 Jon McPhalen
''   E-mail..... jon@jonmcphalen.com
''   Started.... 05 AUG 2010 
''   Updated.... 18 AUG 2010
''
'' =================================================================================================


con

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000  

  CLK_FREQ = ((_clkmode - xtal1) >> 6) * _xinfreq
  MS_001   = CLK_FREQ / 1_000
  US_001   = CLK_FREQ / 1_000_000 


con

  RX1   = 31                                                    ' programming/debug
  TX1   = 30
  SDA   = 29                                                    ' i2c
  SCL   = 28

  START = 15                                                    ' button or PIR

  #0, WICK1, WICK2, WICK3, WICK4, WICK5, WICK6, WICK7, WICK8    ' wick outputs


obj

  leds : "jm_bam8"                                              ' led dimmer
  

var

  long  lottery                                                 ' random #
  long  flickerdelay                                            ' controls flicker speed
  long  stack[64]                                               ' for flicker cog
  
  byte  level[8]                                                ' wick brightness control


pub main | ch, bright, debounce, delay

  lottery := cnt                                                ' seed random #

  leds.init(WICK1)                                              ' start LED (BAM) driver

  bytefill(@level, 0, 8)                                        ' all off to start
  cognew(flicker(3), @stack)                                    ' start flicker generator

  repeat ch from 0 to 2                                         ' check candles
    repeat bright from 0 to 255                                 ' ramp up for fun
      pause(4)
      level[ch] := leds.ezlog(bright)                           ' smoothe ramping with ezlog()

  pause(2000)
  

  ' main loop

  repeat
    repeat bright from 255 to 64                                ' dim down
      pause(4)
      bytefill(@level, leds.ezlog(bright), 3)                   ' adjust wick brightness
      flickerdelay := (255 - bright) * 52 / 10                  ' slow movement while dimming
      
    waitstart(150)                                              ' wait for start input  

    ?lottery                                                    ' re-stir random #
    delay := (||lottery // 8001) + 2000                         ' wait 2 to 10 seconds
    pause(delay)

    flickerdelay := 0                                           ' fast flickering
    repeat ch from 0 to 2                                       ' light candles in order                                      '
      level[ch] := 255                                          '  to full brightness
      pause(750)

    pause(30000)                                                ' hold 30 seconds
  
 
pub pause(ms) | t

'' Delay program ms milliseconds

  t := cnt - 1088                                               ' sync with system counter
  repeat (ms #> 0)                                              ' delay must be > 0
    waitcnt(t += MS_001)


pub waitstart(ms) | debounce

'' Waits for start input to be active at least ms milliseconds

  ms := 0 #> ms <# 1_000                                        ' limit to one second

  debounce := 0
  repeat until (debounce => ms)
    pause(1)
    debounce := ++debounce * ina[START]
  

pri flicker(wix) | lotto, idx, tmp, {
            } target[8], base[8], ramp[8], direction

'' Candle flicker generator
'' -- launch into separate cog

  lotto := cnt                                                  ' seed random #
  repeat idx from 0 to 7
    target[idx] := ?lotto & $FF | $80                           ' new target, 128 to 255 
    base[idx] := 0
    ramp[idx] := (?lotto & %11) + 1                             ' set intial ramp, 1 - 4
  direction := %00000000                                        ' ramp up to start
          
  repeat
    ?lotto                                           
    repeat idx from 0 to (wix-1)                                ' loop through channels
      if (--ramp[idx] == 0)                                     ' dec ramp, check for updates
        ramp[idx] := (?lottery & %11) + 1                       ' set new ramp  
        if (base[idx] < target[idx])                            ' base less than target?                    
          ++base[idx]                                           '   yes, inc base
        elseif (base[idx] > target[idx])                        ' base greater than target?
          --base[idx]                                           '   yes, dec base
        else                                                    ' at target
          direction := togglebit(direction, idx)                ' flip ramp direction
          if (getbit(direction, idx) == 0)                      ' going up?
            target[idx] := ?lotto & $FF | $80 
          else
            target[idx] := ?lotto & $7F 
        
        tmp := (base[idx] * level[idx]) / 255                   ' calculate final base brightess              
        leds.set(idx, tmp)                                      ' set base LED

    if (flickerdelay > 0)
      tmp := (flickerdelay * US_001) #> 400                     ' prevent waitcnt hang
      waitcnt(tmp + cnt)                    '  


pub getbit(value, pos)

'  Returns value.pos (0 or 1)

  return (value >> pos) & 1 


pub setbit(value, pos, bit)

' returns value with bit.0 in value.pos

  if ((bit & 1) == 1)
    return value | (1 << pos)
  else
    return value & !(1 << pos)


pub togglebit(value, pos)

' returns value with value.pos flipped

  return value ^ (1 << pos)

                      
dat

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}